@@ -4,15 +4,18 @@
 """
 
 from . import actions
+from . import action_permissions
 from . import api_log
 from . import authentication
 from . import comments
 from . import datatypes
 from . import errors
+from . import export
 from . import favorites
 from . import files
 from . import groups
 from . import instruments
+from . import instrument_log_entries
 from . import ldap
 from . import locations
 from . import notifications
@@ -41,15 +44,18 @@ __author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
 
 __all__ = [
     'actions',
+    'action_permissions',
     'api_log',
     'authentication',
     'comments',
     'datatypes',
     'errors',
+    'export',
     'favorites',
     'files',
     'groups',
     'instruments',
+    'instrument_log_entries',
     'ldap',
     'locations',
     'notifications',
